program KEM80;

{$mode objfpc}{$H+}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes, SysUtils, CustApp,
  modul_image, Emulator,
  modul_cpu, modul_ram, modul_pio, wincpm, BitOp;


const
  Versions_Nr = '1.0';

type

  { Konsole_EM80 }

  Konsole_EM80 = class(TCustomApplication)
  protected
    procedure DoRun; override;
  public
    kein_Fehler : Boolean;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
  end;

{ Konsole_EM80 }


var
  Application: Konsole_EM80;



constructor Konsole_EM80.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  StopOnException:=True;

  // Initialisieren der anderen Module
  mod_image := Tmod_image.Create;
  mod_cpu := Tmod_cpu.Create;
  mod_ram := Tmod_ram.Create;
  mod_pio := Tmod_pio.Create;

  // IMAGE
  wcpm := Twcpm.Create;
  wcpm.eigener_Name:=ParamStr(0);
  wcpm.Write_Log_File:=false;
  wcpm.load_dpb_img;

  Writeln(' ');
  Writeln('KEM80 '+Versions_Nr+' - CP/M Konsolen-EMulator fuer Z80');
  Writeln('(c) by Ronald Daleske');


  if DirectoryExists(CPM_Laufwerk) then begin

    kein_Fehler:=true;

    wcpm.Import_Files_To_Image;
    // nach der Initialisiereung
    mod_image.Lade_BIOS_in_RAM;
    mod_cpu.Reg.SRS.PC:=$00;

  end
  else begin
    kein_Fehler:=false;
    Writeln('Fehler: das CP/M-Verzeichnis <'+CPM_Laufwerk+'> konnte nicht gefunden werden ...');
  end; { if DirectoryExists(CPM_Laufwerk) then }



end; { constructor Konsole_EM80.Create }





procedure Konsole_EM80.DoRun;
var
  P_Zaehler : Integer;
begin

  { add your program here }



  mod_pio.CONIN_Buffer:='';
  for P_Zaehler := 1 to ParamCount do begin
    mod_pio.CONIN_Buffer:=mod_pio.CONIN_Buffer+ParamStr(P_Zaehler);

    if ParamCount>P_Zaehler then
      mod_pio.CONIN_Buffer:=mod_pio.CONIN_Buffer+' '
    else
      mod_pio.CONIN_Buffer:=mod_pio.CONIN_Buffer+CHR($0D)+CHR($0A);

  end; { for P_Zaehler := 1 to ParamCount do }


  {
  // Testbetrieb, Dummy-Parameter
  if ParamCount=0 then begin
    mod_pio.CONIN_Buffer:='M80 CPD7.ERL,CPD7.PRN=CPD7BAT/M/Z'+CHR($0D)+CHR($0A);
  end;
   }


  if kein_Fehler then begin

    if ParamCount>0 then begin
      Z80.Schleife_abarbeiten;
    end { if ParamCount=0 then }
    else begin
      Writeln('Fehler: kein Parameter uebergeben...');
    end; { if ParamCount>0 then }

  end; { if kein_Fehler then }




  // stop program loop
  Terminate;
end; { procedure Konsole_EM80.DoRun }



destructor Konsole_EM80.Destroy;
begin

//  if kein_Fehler then
    wcpm.Export_Files_From_Image;

  wcpm.Free;

  Z80.Free;

  mod_image.Free;
  mod_cpu.Free;
  mod_ram.Free;
  mod_pio.Free;

  Writeln(' ');

  inherited Destroy;
end; { destructor Konsole_EM80.Destroy }




{$R *.res}

begin
  Application:=Konsole_EM80.Create(nil);
  Application.Title:='EM80 mit Konsole';
  Application.Run;
  Application.Free;
end.


